<?xml version='1.0' encoding='utf-8' ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns:xalan="http://xml.apache.org/xalan"
		xmlns:in="http://www.ibm.com/xmlns/prod/websphere/monitoring/6.0.2/mm"
		xmlns:mm="http://www.ibm.com/xmlns/prod/websphere/monitoring/6.1.0/mm"
		xmlns:mutil="xalan://com.ibm.wbimonitor.xml.migration.util.MigrationUtils"
  		extension-element-prefixes="mutil">
  		
	<!-- kpiModel -->
	<xsl:template match="in:monitor/kpiModel">
		<xsl:element name="kpiModel">
			<xsl:copy-of select="@*"/>
			<xsl:if test="description">
				<xsl:element name="description">
					<xsl:value-of select="description"/>
				</xsl:element>
			</xsl:if>
			<xsl:for-each select="kpiContext">
				<xsl:element name="kpiContext">
					<xsl:copy-of select="@*"/>
					<xsl:variable name="kpiRangesIds">
						<xsl:for-each select="kpi">
							<xsl:if test="ranges">
								<xsl:value-of select="'/'"/>
								<xsl:value-of select="@id"/>
								<xsl:value-of select="'/'"/>
								<xsl:value-of select="ranges/@id"/>																					
							</xsl:if>
						</xsl:for-each>
					</xsl:variable>
					<xsl:variable name="dummy">
						<xsl:value-of select="mutil:cacheKPIRangesId(@id, $kpiRangesIds)"/>
					</xsl:variable>
					<xsl:if test="description">
						<xsl:element name="description">
							<xsl:value-of select="description"/>
						</xsl:element>
					</xsl:if>
					<xsl:apply-templates select="trigger"/>
					<xsl:apply-templates select="inboundEvent"/>
					<xsl:apply-templates select="outboundEvent"/>
					<xsl:apply-templates select="kpi"/>
	      			</xsl:element>
	   		 </xsl:for-each>
	   	</xsl:element>
	</xsl:template>
	
	<!-- handle kpi trigger (check gatingCondition expression to see if it contains Ranges id)-->
	<xsl:template match="trigger">
		<xsl:element name="trigger">
			<xsl:copy-of select="@*"/>	
			<xsl:if test="description">
				<xsl:element name="description">
					<xsl:value-of select="description"/>
				</xsl:element>
			</xsl:if>
			<xsl:for-each select="onTrigger">
				<xsl:element name="onTrigger">
					<xsl:copy-of select="@*"/>
	      		</xsl:element>
	      	</xsl:for-each>
	      	<xsl:for-each select="evaluationTime">
				<xsl:element name="evaluationTime">
					<xsl:copy-of select="@*"/>
      			</xsl:element>	      		
      		</xsl:for-each>
	      	<xsl:for-each select="onValueChange">
				<xsl:element name="onValueChange">
					<xsl:copy-of select="@*"/>
	      		</xsl:element>
			</xsl:for-each>
		    <xsl:for-each select="onEvent">      							
		    	<xsl:element name="onEvent">
					<xsl:copy-of select="@*"/>
		    	</xsl:element>
		    </xsl:for-each>
		    <xsl:if test="gatingCondition">
				<xsl:element name="gatingCondition">
					<xsl:attribute name="expression">
						<xsl:variable name="tempGatingCondition">
							<xsl:value-of select="mutil:convertDuration(gatingCondition/@expression)"/>
						</xsl:variable>
						<xsl:value-of select="mutil:removeRanges($tempGatingCondition)"/>
					</xsl:attribute>
				</xsl:element>
			</xsl:if>
	      </xsl:element>
	</xsl:template>
	
	<!-- handle outbound event (check filter expression, map/output/assignment values to see if it contains Ranges id)-->
	<xsl:template match="outboundEvent">
		<xsl:element name="outboundEvent">
			<xsl:copy-of select="@*[not(name()='type')]"/>
			<xsl:variable name="origType">
				<xsl:value-of select="@type" />
			</xsl:variable>
			<xsl:attribute name="rootElement">cbe:CommonBaseEvent</xsl:attribute>
			<xsl:attribute name="extensionName">
				<xsl:value-of select="mutil:getCBEExtensionName($origType)"/>
			</xsl:attribute>	
			<xsl:if test="description">
				<xsl:element name="description">
					<xsl:value-of select="description"/>
				</xsl:element>
			</xsl:if>
			<xsl:for-each select="map">
				<xsl:element name="map">
					<xsl:if test="trigger">
						<xsl:element name="trigger">
							<xsl:copy-of select="trigger/@*"/>
      					</xsl:element>
					</xsl:if>		
					<xsl:element name="outputValue">
						<xsl:if test="outputValue/singleValue">
							<xsl:element name="singleValue">
								<xsl:attribute name="expression">
									<xsl:value-of select="mutil:convertDuration(outputValue/singleValue/@expression)"/>
								</xsl:attribute>
				      		</xsl:element>
						</xsl:if>
						<xsl:if test="outputValue/assignments">
							<xsl:element name="assignments">
								<xsl:for-each select="outputValue/assignments/assignment">
									<xsl:element name="assignment">
										<xsl:attribute name="leftValue">
											<xsl:value-of select="mutil:convertDuration(@leftValue)"/>
										</xsl:attribute>
										<xsl:attribute name="rightValue">
											<xsl:variable name="tempAssignRightValue">
												<xsl:value-of select="mutil:convertDuration(@rightValue)"/>												
											</xsl:variable>
											<xsl:value-of select="mutil:removeRanges($tempAssignRightValue)"/>
										</xsl:attribute>
									</xsl:element>
								</xsl:for-each>
	      					</xsl:element>
						</xsl:if>
					</xsl:element>
    		  	</xsl:element>
		   	</xsl:for-each>
		   	<xsl:if test="filter">
				<xsl:element name="filter">
					<xsl:attribute name="expression">
						<xsl:variable name="tempOutboundFilter">
							<xsl:value-of select="mutil:convertDuration(filter/@expression)"/>
						</xsl:variable>
						<xsl:value-of select="mutil:removeRanges($tempOutboundFilter)"/>
					</xsl:attribute>
				</xsl:element>
			</xsl:if>
   		</xsl:element>
	</xsl:template>
	
	<!-- kpi inboundEvent copying -->
	<xsl:template match="inboundEvent">
		<xsl:element name="inboundEvent">
			<xsl:copy-of select="@*[not(name()='type')]"/>
			<xsl:variable name="origType">
				<xsl:value-of select="@type" />
			</xsl:variable>
			<xsl:attribute name="rootElement">cbe:CommonBaseEvent</xsl:attribute>
			<xsl:attribute name="extensionName">
				<xsl:value-of select="mutil:getCBEExtensionName($origType)"/>
			</xsl:attribute>
			<xsl:if test="description">
				<xsl:element name="description">
					<xsl:value-of select="description"/>
				</xsl:element>
			</xsl:if>
			<xsl:if test="correlationPredicate">
				<xsl:element name="correlationPredicate">
					<xsl:attribute name="expression">
						<xsl:value-of select="mutil:convertDuration(correlationPredicate/@expression)"/>
					</xsl:attribute>
		    	</xsl:element>
			</xsl:if>						
			<xsl:if test="filter">
				<xsl:element name="filter">
					<xsl:attribute name="expression">
						<xsl:value-of select="mutil:convertDuration(filter/@expression)"/>
					</xsl:attribute>
		    	</xsl:element>
			</xsl:if>
		</xsl:element>
	</xsl:template>
	
	<!-- kpi migration -->
	<xsl:template match="kpi">
		<xsl:element name="kpi">
			<xsl:copy-of select="@*"/>
			<!-- move old ranges/@rangeType up to kpi/@rangeType -->
			<xsl:choose>
				<xsl:when test="ranges">
					<xsl:attribute name="rangeType">
						<xsl:value-of select="ranges/@rangeType"/>
					</xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="rangeType">
						<xsl:value-of select="'actualValue'"/>
					</xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="description">
				<xsl:element name="description">
					<xsl:value-of select="description"/>
				</xsl:element>
			</xsl:if>
			<xsl:element name="aggregatedDefinition">
				<!-- no cube element anymore, we need to keep cube/@ref information -->
				<xsl:variable name="cubeRef">
					<xsl:value-of select="definition/cube/@ref" />
				</xsl:variable>
				<xsl:variable name="measureRef">
					<xsl:value-of select="definition/measure/@ref" />
				</xsl:variable>
				<xsl:for-each select="/in:monitor/dataMartModel/cube">
					<xsl:if test="@id = $cubeRef">
						<xsl:variable name="measureSource">
							<xsl:for-each select="measure">
								<xsl:if test="@id = $measureRef">
									<xsl:value-of select="@source"/>
								</xsl:if>
							</xsl:for-each>
						</xsl:variable>
						<xsl:variable name="measureAggregationType">
							<xsl:for-each select="measure">
								<xsl:if test="@id = $measureRef">
									<xsl:value-of select="@aggregationType"/>
								</xsl:if>
							</xsl:for-each>
						</xsl:variable>
						<xsl:variable name="factTableId">
							<xsl:value-of select="mutil:getFactTableIdFromMeasureSource($measureSource)" />
						</xsl:variable>
						<xsl:variable name="factAttributeId">
							<xsl:value-of select="mutil:getFactAttributeIdFromMeasureSource($measureSource)" />
						</xsl:variable>
						<!-- get aggregationType from measure -->
						<xsl:if test="$measureAggregationType != ''">
							<xsl:attribute name="aggregationType">
								<xsl:value-of select="$measureAggregationType"/>	
							</xsl:attribute>
						</xsl:if>
						<!-- define versionAgggregation default to "allVersions" -->
						<xsl:attribute name="versionAggregation">allVersions</xsl:attribute>
						<!-- directly reference monitoringContext -->
						<xsl:element name="monitoringContext">
							<xsl:attribute name="ref">
								<xsl:value-of select="@monitoringContext"/>
							</xsl:attribute>
						</xsl:element>
						<!-- directly reference metric under the monitoringContext -->
						<xsl:element name="metric">
							<xsl:attribute name="ref">
								<xsl:if test="factTable/@id = $factTableId">
									<xsl:for-each select="factTable/attribute">
										<xsl:if test="@id = $factAttributeId">
											<xsl:value-of select="@attributeSource"/>	
										</xsl:if>
									</xsl:for-each>
								</xsl:if>
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
				</xsl:for-each>
				
				
				<xsl:for-each select="definition/dimension">
					<xsl:for-each select="attribute">
						<xsl:variable name="attrRef1">
							<xsl:value-of select="mutil:getKPIDefDimensionAttrRef1(@ref)" />
						</xsl:variable>
						<xsl:variable name="attrRef2">
							<xsl:value-of select="mutil:getKPIDefDimensionAttrRef2(@ref)" />
						</xsl:variable>
						<!-- get metricFilter/@ref from dataMartModel/dimension/attribute/@attributeSource using old cube/@ref and attribute/@ref -->
						<xsl:element name="metricFilter">
							<xsl:attribute name="ref">
								<xsl:variable name="metricRef">
									<xsl:for-each select="/in:monitor/dataMartModel/cube[@id = $cubeRef]">
										<xsl:for-each select="dimension[@id = $attrRef1]">
											<xsl:for-each select="attribute[@id = $attrRef2]">
												<xsl:value-of select="@attributeSource" />
											</xsl:for-each>
										</xsl:for-each>
									</xsl:for-each>
								</xsl:variable>
								<xsl:value-of select="$metricRef"/> 
							</xsl:attribute>
							<xsl:attribute name="operator">equals</xsl:attribute>
							<xsl:attribute name="isCaseSensitive">true</xsl:attribute>
							<xsl:element name="value">
								<xsl:attribute name="expression">
									<xsl:variable name="tempExpressionValue">
										<xsl:value-of select="mutil:getExpressionValue(@value)"/>
									</xsl:variable>
									<xsl:value-of select="mutil:convertDuration($tempExpressionValue)"/>
								</xsl:attribute>
							</xsl:element>
						</xsl:element>
					</xsl:for-each>
				</xsl:for-each>
				
				<xsl:if test="definition/timeDimension">
					<xsl:element name="dateTimeMetricFilter">
						<xsl:variable name="timeDimensionRef">
							<xsl:value-of select="definition/timeDimension/@ref" />
						</xsl:variable>
						<xsl:attribute name="ref">
							<xsl:variable name="dtMetricRef">
								<xsl:for-each select="/in:monitor/dataMartModel/cube[@id = $cubeRef]">
									<xsl:for-each select="dimension[@id = $timeDimensionRef]">
											<xsl:value-of select="attribute/@attributeSource" />
									</xsl:for-each>
								</xsl:for-each>
							</xsl:variable>
							<xsl:value-of select="$dtMetricRef"/> 
						</xsl:attribute>
						<xsl:if test="definition/timeDimension/repeatingPeriod">
							<xsl:element name="repeatingPeriod">
								<xsl:copy-of select="definition/timeDimension/repeatingPeriod/@*"/>
								<xsl:attribute name="timezone">
									<xsl:value-of select="mutil:getDefaultTimezone()" />
								</xsl:attribute>
							</xsl:element>
						</xsl:if>
						<xsl:if test="definition/timeDimension/rollingPeriod">
							<xsl:element name="rollingPeriod">
								<!-- copy value of numDays -->
								<xsl:if test="definition/timeDimension/rollingPeriod/@numDays">
									<xsl:attribute name="numPeriods">
										<xsl:value-of select="definition/timeDimension/rollingPeriod/@numDays" />
									</xsl:attribute>
									<!-- by default it is days -->
									<xsl:attribute name="periodType">days</xsl:attribute>
								</xsl:if>
							</xsl:element>
						</xsl:if>
						<xsl:if test="definition/timeDimension/fixedPeriod">
							<xsl:element name="fixedPeriod">
								<xsl:if test="definition/timeDimension/fixedPeriod/@startDate">
									<xsl:if test="definition/timeDimension/fixedPeriod/@endDate">
										<xsl:attribute name="startDate">
											<xsl:value-of select="mutil:convertDateToDateTime(definition/timeDimension/fixedPeriod/@startDate)" />
										</xsl:attribute>								
										<xsl:attribute name="endDate">
											<xsl:value-of select="mutil:convertDateToDateTime(definition/timeDimension/fixedPeriod/@endDate)" />
										</xsl:attribute>
										<xsl:attribute name="timezone">
											<xsl:value-of select="mutil:getDefaultTimezone()" />
										</xsl:attribute>
									</xsl:if>
								</xsl:if>
							</xsl:element>
						</xsl:if>
					</xsl:element>
				</xsl:if>
			</xsl:element>
			<xsl:if test="target">
				<xsl:element name="target">
					<xsl:copy-of select="target/@*"/>
					<xsl:if test="target/description">
						<xsl:if test="target/description[normalize-space(.)] != ''">
							<xsl:element name="description">
								<xsl:copy-of select="target/description/node()"/>
	      					</xsl:element>
	      				</xsl:if>
					</xsl:if>
	      		</xsl:element>
			</xsl:if>
			<xsl:if test="ranges">
				<xsl:for-each select="ranges/range">
					<xsl:element name="range">
						<xsl:copy-of select="@*"/>
						<xsl:if test="description">
							<xsl:element name="description">
								<xsl:value-of select="description"/>
							</xsl:element>
						</xsl:if>
						<xsl:element name="startValue">
							<xsl:copy-of select="startValue/@*"/>
							<xsl:if test="startValue/description">
								<xsl:if test="startValue/description[normalize-space(.)] != ''">
									<xsl:element name="description">
										<xsl:copy-of select="startValue/description/node()"/>
									</xsl:element>
								</xsl:if>
							</xsl:if>
						</xsl:element>
						<xsl:element name="endValue">
							<xsl:copy-of select="endValue/@*"/>
							<xsl:if test="endValue/description">
								<xsl:if test="endValue/description[normalize-space(.)] != ''">
									<xsl:element name="description">
										<xsl:copy-of select="endValue/description/node()"/>
									</xsl:element>
								</xsl:if>
							</xsl:if>
						</xsl:element>
	      			</xsl:element>
	   			</xsl:for-each>
			</xsl:if>
	      </xsl:element>
	</xsl:template>
	   		
</xsl:stylesheet>