<?xml version='1.0' encoding='utf-8' ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns:xalan="http://xml.apache.org/xalan"
		xmlns:in="http://www.ibm.com/xmlns/prod/websphere/monitoring/6.0.2/mm"
		xmlns:mm="http://www.ibm.com/xmlns/prod/websphere/monitoring/6.1.0/mm"
		xmlns:mutil="xalan://com.ibm.wbimonitor.xml.migration.util.MigrationUtils"
  		extension-element-prefixes="mutil">
	  	
	<!-- monitorDetailsModel -->
	<xsl:template match="in:monitor/monitorDetailsModel">
		<xsl:element name="monitorDetailsModel">
			<xsl:copy-of select="@*"/>
			<xsl:if test="description">
				<xsl:element name="description">
					<xsl:value-of select="description"/>
				</xsl:element>
			</xsl:if>
			<xsl:apply-templates select="monitoringContext"/>
   		</xsl:element>
	</xsl:template>
	
	<!-- monitoringContext // create a template for recursive usage -->
	<xsl:template match="monitoringContext">
		<xsl:element name="monitoringContext">
			<xsl:copy-of select="@*"/>
			<xsl:if test="../@id = 'MDM'">
				<xsl:attribute name="eventSequenceIDPath">*/predefinedData/creationTime</xsl:attribute>
			</xsl:if>
			<xsl:if test="description">
				<xsl:element name="description">
					<xsl:value-of select="description"/>
				</xsl:element>
			</xsl:if>
			
			<!-- start copying trigger -->
		   	<xsl:for-each select="trigger">
				<xsl:element name="trigger">
					<xsl:copy-of select="@*"/>	
					<xsl:if test="description">
						<xsl:element name="description">
							<xsl:value-of select="description"/>
						</xsl:element>
					</xsl:if>
					<xsl:for-each select="onTrigger">
						<xsl:element name="onTrigger">
							<xsl:copy-of select="@*"/>
		    			</xsl:element>
		    		</xsl:for-each>
		    		<xsl:for-each select="evaluationTime">
						<xsl:element name="evaluationTime">
							<xsl:copy-of select="@*"/>
		      			</xsl:element>
		      		</xsl:for-each>
		      		<xsl:for-each select="onValueChange">
						<xsl:element name="onValueChange">
							<xsl:copy-of select="@*"/>
		      			</xsl:element>
					</xsl:for-each>
		      		<xsl:for-each select="onEvent">      							
		      			<xsl:element name="onEvent">
							<xsl:copy-of select="@*"/>
		      			</xsl:element>
		      		</xsl:for-each>
		      		<xsl:if test="gatingCondition">
						<xsl:element name="gatingCondition">
							<xsl:attribute name="expression">
								<xsl:value-of select="mutil:convertDuration(gatingCondition/@expression)"/>
							</xsl:attribute>
			      		</xsl:element>
					</xsl:if>
		      	</xsl:element>
		   	</xsl:for-each>
		   	 <!-- end copying trigger -->
		   	 
			<!--  start copying inbound event -->
	      	<xsl:for-each select="inboundEvent">
				<xsl:element name="inboundEvent">
					<xsl:copy-of select="@*[not(name()='type')]"/>
					<xsl:variable name="origType">
						<xsl:value-of select="@type" />
					</xsl:variable>
					<xsl:attribute name="rootElement">cbe:CommonBaseEvent</xsl:attribute>
					<xsl:attribute name="extensionName">
						<xsl:value-of select="mutil:getCBEExtensionName($origType)"/>
					</xsl:attribute>
					<xsl:if test="description">
						<xsl:element name="description">
							<xsl:value-of select="description"/>
						</xsl:element>
					</xsl:if>
					<xsl:if test="correlationPredicate">
						<xsl:element name="correlationPredicate">
							<xsl:attribute name="expression">
								<xsl:value-of select="mutil:convertDuration(correlationPredicate/@expression)"/>
							</xsl:attribute>
		    			</xsl:element>
					</xsl:if>						
					<xsl:if test="filter">
						<xsl:element name="filter">
							<xsl:attribute name="expression">
								<xsl:value-of select="mutil:convertDuration(filter/@expression)"/>
							</xsl:attribute>
		    			</xsl:element>
					</xsl:if>
		    	</xsl:element>
	   		</xsl:for-each>
	   		<!-- end copying inbound event -->
		   	 
		   	 <!-- start copying outboundEvent -->
	   		 <xsl:for-each select="outboundEvent">
				<xsl:element name="outboundEvent">
					<xsl:copy-of select="@*[not(name()='type')]"/>
					<xsl:variable name="origType">
						<xsl:value-of select="@type" />
					</xsl:variable>
					<xsl:attribute name="rootElement">cbe:CommonBaseEvent</xsl:attribute>
					<xsl:attribute name="extensionName">
						<xsl:value-of select="mutil:getCBEExtensionName($origType)"/>
					</xsl:attribute>	
					<xsl:if test="description">
						<xsl:element name="description">
							<xsl:value-of select="description"/>
						</xsl:element>
					</xsl:if>
					<xsl:apply-templates select="map"/>
	   		 		<xsl:if test="filter">
						<xsl:element name="filter">
							<xsl:attribute name="expression">
								<xsl:value-of select="mutil:convertDuration(filter/@expression)"/>
							</xsl:attribute>
		      			</xsl:element>
					</xsl:if>
	      		</xsl:element>
	   		 </xsl:for-each>
	   		 <!-- end copying outboundEvent -->
	   		 
	   		 <!-- cache metrics that used in datamart dimension -->
			<xsl:for-each select="/in:monitor/dataMartModel/cube">
				<xsl:for-each select="dimension">
					<xsl:for-each select="attribute">
						<xsl:variable name="dummy">
							<xsl:value-of select="mutil:cacheMetricDimensional(@attributeSource)"/>
						</xsl:variable>
					</xsl:for-each>
				</xsl:for-each>
			</xsl:for-each>
				
	   		 <!-- start copying metrics -->
		   	<xsl:for-each select="metric">
				<xsl:element name="metric">
					<xsl:copy-of select="@*"/>
					<xsl:attribute name="valueRequired">
						<xsl:value-of select="mutil:isMetricUsedDimension(@id)"/>
					</xsl:attribute>
					<xsl:variable name="datatype">
						<xsl:value-of select="@type" />
					</xsl:variable>
					<xsl:if test="description">
						<xsl:element name="description">
							<xsl:value-of select="description"/>
						</xsl:element>
					</xsl:if>
					<xsl:apply-templates select="map"/>
					<xsl:if test="defaultValue">
						<xsl:element name="defaultValue">
							<xsl:if test="defaultValue/singleValue">
								<xsl:element name="singleValue">
									<xsl:attribute name="expression">
										<xsl:value-of select="mutil:convertDuration(defaultValue/singleValue/@expression)"/>
									</xsl:attribute>
				      			</xsl:element>
							</xsl:if>
							<xsl:if test="defaultValue/assignments">
								<xsl:element name="assignments">
									<xsl:for-each select="defaultValue/assignments/assignment">
										<xsl:element name="assignment">
											<xsl:attribute name="leftValue">
												<xsl:value-of select="mutil:convertDuration(@leftValue)"/>
											</xsl:attribute>
											<xsl:attribute name="rightValue">
												<xsl:value-of select="mutil:convertDuration(@rightValue)"/>
											</xsl:attribute>
										</xsl:element>
									</xsl:for-each>
	      						</xsl:element>
							</xsl:if>
						</xsl:element>
					</xsl:if>
	      		</xsl:element>
		   	</xsl:for-each>
		   	<!-- end copying metrics -->
	   		 
	   		 <!-- start copying counter -->
	   		 <xsl:for-each select="counter">
				<xsl:element name="counter">
					<xsl:copy-of select="@*"/>
					<xsl:if test="description">
						<xsl:element name="description">
							<xsl:value-of select="description"/>
						</xsl:element>
					</xsl:if>
					<xsl:for-each select="incrementedWhen">
						<xsl:element name="incrementedWhen">
							<xsl:copy-of select="@*"/>
	      				</xsl:element>
	   		 		</xsl:for-each>
	   		 		<xsl:for-each select="decrementedWhen">
						<xsl:element name="decrementedWhen">
							<xsl:copy-of select="@*"/>
	      				</xsl:element>
	   		 		</xsl:for-each>
	   		 		<xsl:for-each select="setToZeroWhen">
						<xsl:element name="setToZeroWhen">
							<xsl:copy-of select="@*"/>
	      				</xsl:element>
	   		 		</xsl:for-each>
	      		</xsl:element>
	   		 </xsl:for-each>
	   		 <!-- end copying counter -->
	   		 
	   		 <!-- start copying stopwatch -->
	   		 <xsl:for-each select="stopwatch">
				<xsl:element name="stopwatch">
					<xsl:copy-of select="@*"/>
					<xsl:if test="description">
						<xsl:element name="description">
							<xsl:value-of select="description"/>
						</xsl:element>
					</xsl:if>
					<xsl:for-each select="startedWhen">
						<xsl:element name="startedWhen">
							<xsl:copy-of select="@*"/>
	      				</xsl:element>
	   		 		</xsl:for-each>
	   		 		<xsl:for-each select="stoppedWhen">
						<xsl:element name="stoppedWhen">
							<xsl:copy-of select="@*"/>
	      				</xsl:element>
	   		 		</xsl:for-each>
	   		 		<xsl:for-each select="resetWhen">
						<xsl:element name="resetWhen">
							<xsl:copy-of select="@*"/>
	      				</xsl:element>
	   		 		</xsl:for-each>	
	      		</xsl:element>
	   		 </xsl:for-each>
	   		 <!-- end copying stopwatch -->
	   		 
	   		 <!-- recursively apply monitor context -->
			<xsl:apply-templates select="monitoringContext"/>
   		</xsl:element>
	</xsl:template>
	
	<xsl:template match="map">
		<xsl:element name="map">
			<xsl:if test="description">
				<xsl:element name="description">
					<xsl:value-of select="description"/>
				</xsl:element>
			</xsl:if>
			<xsl:if test="trigger">
				<xsl:element name="trigger">
					<xsl:copy-of select="trigger/@*"/>
	      		</xsl:element>
			</xsl:if>
			<xsl:if test="outputValue">
				<xsl:element name="outputValue">
					<xsl:if test="outputValue/singleValue">
						<xsl:element name="singleValue">
							<xsl:attribute name="expression">
								<xsl:value-of select="mutil:convertDuration(outputValue/singleValue/@expression)"/>
							</xsl:attribute>
			      		</xsl:element>
					</xsl:if>
					<xsl:if test="outputValue/assignments">
						<xsl:element name="assignments">
							<xsl:for-each select="outputValue/assignments/assignment">
								<xsl:element name="assignment">
									<xsl:attribute name="leftValue">
										<xsl:value-of select="mutil:convertDuration(@leftValue)"/>
									</xsl:attribute>
									<xsl:attribute name="rightValue">
										<xsl:value-of select="mutil:convertDuration(@rightValue)"/>
									</xsl:attribute>
								</xsl:element>
							</xsl:for-each>
			      		</xsl:element>
					</xsl:if>
		      	</xsl:element>
		      </xsl:if>
	    </xsl:element>
	</xsl:template>

</xsl:stylesheet>